"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const auth = require("../auth");
const TestServer_1 = require("../TestServer");
const services = require("../services");
const chai_1 = require("chai");
const chai = require("chai");
const superagent = require("superagent");
const Realm = require("realm");
const uuid = require("uuid");
const path = require("path");
const faker = require("faker");
chai.use(require("chai-as-promised"));
describe('Realm-JS integration', function () {
    const ProductSchema = {
        name: 'Product',
        primaryKey: 'productId',
        properties: {
            productId: { type: 'int', optional: false },
            name: { type: 'string', optional: false },
            price: { type: 'float', optional: false }
        }
    };
    let server;
    before(() => __awaiter(this, void 0, void 0, function* () {
        server = new TestServer_1.TestServer();
        const authService = new services.AuthService();
        authService.addProvider(new auth.PasswordAuthProvider({ autoCreateAdminUser: true, saltWorkIterations: 1 }));
        server.addService(authService);
        server.addService(new services.RealmDirectoryService());
        server.addService(new services.PermissionService());
        server.addService(new services.SyncProxyService());
        server.addService(new services.SyncService());
        yield server.start();
    }));
    after(() => __awaiter(this, void 0, void 0, function* () {
        if (server) {
            yield server.shutdown();
        }
    }));
    it('should be able to register to realm-js', function () {
        return __awaiter(this, void 0, void 0, function* () {
            yield chai_1.assert.isFulfilled(Realm.Sync.User.register(server.url, 'sampleuser', 'samplepassword'));
        });
    });
    it('should be able to login to realm-js', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const registeredUser = yield Realm.Sync.User.register(server.url, 'newuser', 'samplepassword');
            registeredUser.logout();
            yield chai_1.assert.isFulfilled(Realm.Sync.User.login(server.url, 'newuser', 'samplepassword'));
        });
    });
    it('should be able to login to realm-js with an empty string password', function () {
        return __awaiter(this, void 0, void 0, function* () {
            yield chai_1.assert.isFulfilled(Realm.Sync.User.login(server.url, 'realm-admin', ''));
        });
    });
    it('should create error when trying to login with an unknown username', function () {
        return __awaiter(this, void 0, void 0, function* () {
            yield chai_1.assert.isRejected(Realm.Sync.User.login(server.url, 'sameuser', 'samplepassword'), 'The provided credentials are invalid or the user does not exist.');
        });
    });
    it('should create error when trying to register an existing username', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const registeredUser = yield Realm.Sync.User.register(server.url, 'sameuser', 'samplepassword');
            registeredUser.logout();
            yield chai_1.assert.isRejected(Realm.Sync.User.register(server.url, 'sameuser', 'samplepassword'), 'The provided credentials are invalid or the user does not exist.');
        });
    });
    function registerAndGetRealm(username, password, localPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = yield Realm.Sync.User.register(server.url, username, password);
            return new Realm({
                path: localPath,
                sync: {
                    url: `realm://${server.address}/~/products`,
                    user,
                },
                schema: [ProductSchema]
            });
        });
    }
    function loginAndGetRealm(username, password, localPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const user = yield Realm.Sync.User.login(server.url, username, password);
            return new Realm({
                path: localPath,
                sync: {
                    url: `realm://${server.address}/~/products`,
                    user,
                },
                schema: [ProductSchema]
            });
        });
    }
    it('should be able to write to a realm', () => __awaiter(this, void 0, void 0, function* () {
        const USERNAME = uuid.v4();
        const localPath1 = path.join(server.config.dataPath, "local1.realm");
        const realm1 = yield registerAndGetRealm(USERNAME, 'samplepassword', localPath1);
        const localPath2 = path.join(server.config.dataPath, "local2.realm");
        const realm2 = yield loginAndGetRealm(USERNAME, 'samplepassword', localPath2);
        return new Promise((resolve, reject) => {
            let obj1;
            realm2.addListener('change', (realm, event) => {
                const obj2 = realm2.objectForPrimaryKey('Product', 0);
                chai_1.assert.equal(obj1.name, obj2.name);
                chai_1.assert.equal(obj1.price, obj2.price);
                resolve();
            });
            realm1.write(() => {
                obj1 = realm1.create('Product', {
                    productId: 0,
                    name: faker.commerce.productName(),
                    price: faker.random.number({ min: 12, max: 7000 })
                }, true);
            });
        });
    }));
    describe("user.applyPermissions", () => {
        let userA;
        let realmA;
        let userB;
        let userBProviderId;
        let realmB;
        beforeEach(() => __awaiter(this, void 0, void 0, function* () {
            userA = yield Realm.Sync.User.register(server.url, faker.internet.userName(), faker.internet.password());
            userBProviderId = faker.internet.userName();
            userB = yield Realm.Sync.User.register(server.url, userBProviderId, faker.internet.password());
            realmA = new Realm({
                sync: {
                    user: userA,
                    url: `realm://${server.address}/~/products`
                },
                schema: [ProductSchema]
            });
            realmB = new Realm({
                sync: {
                    user: userB,
                    url: `realm://${server.address}/~/products`
                },
                schema: [ProductSchema]
            });
        }));
        afterEach(() => {
            realmA.close();
        });
        function assertAppliedPermissions(response, expected) {
            chai_1.assert.equal(response.statusCode, 0);
            chai_1.assert.equal(response.mayRead, expected.mayRead);
            chai_1.assert.equal(response.mayWrite, expected.mayWrite);
            chai_1.assert.equal(response.mayManage, expected.mayManage);
            chai_1.assert.equal(response.userId, expected.userId);
            chai_1.assert.equal(response.realmUrl, expected.realmUrl);
            if (expected.metadataKey) {
                chai_1.assert.equal(response.metadataKey, expected.metadataKey);
            }
            if (expected.metadataValue) {
                chai_1.assert.equal(response.metadataValue, expected.metadataValue);
            }
        }
        function assertUserPermissions(user, expected) {
            return __awaiter(this, void 0, void 0, function* () {
                const result = yield user.getGrantedPermissions();
                const perm = result.slice().filter((p) => {
                    return p.userId === expected.userId;
                }).find((p) => {
                    return p.path === expected.path;
                });
                chai_1.assert.isDefined(perm);
                chai_1.assert.equal(perm.mayRead, expected.mayRead, `Expected user to have mayRead=${expected.mayRead}`);
                chai_1.assert.equal(perm.mayWrite, expected.mayWrite, `Expected user to have mayWrite=${expected.mayWrite}`);
                chai_1.assert.equal(perm.mayManage, expected.mayManage, `Expected user to have mayManage=${expected.mayManage}`);
            });
        }
        describe("with email address", () => {
            let adminUser;
            let userBEmailAddress;
            beforeEach(() => __awaiter(this, void 0, void 0, function* () {
                userBEmailAddress = faker.internet.email();
                adminUser = yield Realm.Sync.User.login(server.url, "realm-admin", "");
                yield superagent.put(`${server.url}/auth/users`).set({
                    Authorization: adminUser.token,
                })
                    .send({
                    provider_id: userBProviderId,
                    provider: "password",
                    metadata: {
                        email: userBEmailAddress
                    },
                    user_id: userB.identity
                });
            }));
            describe("userA granting read to userB", () => {
                it("should be successful", () => __awaiter(this, void 0, void 0, function* () {
                    assertAppliedPermissions(yield userA.applyPermissions({ metadataKey: "email", metadataValue: userBEmailAddress }, "/~/products", 'read'), {
                        mayRead: true,
                        mayWrite: false,
                        mayManage: false,
                        metadataKey: "email",
                        metadataValue: userBEmailAddress,
                        userId: "",
                        realmUrl: "/~/products",
                    });
                    yield assertUserPermissions(userB, {
                        userId: userB.identity,
                        mayRead: true,
                        mayWrite: false,
                        mayManage: false,
                        path: `/${userA.identity}/products`
                    });
                }));
            });
        });
        describe("userA granting read to userB", () => {
            it("should be successful", () => __awaiter(this, void 0, void 0, function* () {
                assertAppliedPermissions(yield userA.applyPermissions({ userId: userB.identity }, "/~/products", 'read'), {
                    mayRead: true,
                    mayWrite: false,
                    mayManage: false,
                    userId: userB.identity,
                    realmUrl: "/~/products",
                });
                yield assertUserPermissions(userA, {
                    userId: userA.identity,
                    mayRead: true,
                    mayWrite: true,
                    mayManage: true,
                    path: `/${userA.identity}/products`
                });
                yield assertUserPermissions(userB, {
                    userId: userB.identity,
                    mayRead: true,
                    mayWrite: false,
                    mayManage: false,
                    path: `/${userA.identity}/products`
                });
            }));
        });
        describe("userA granting write to userB", () => {
            it("should be successful", () => __awaiter(this, void 0, void 0, function* () {
                assertAppliedPermissions(yield userA.applyPermissions({ userId: userB.identity }, "/~/products", 'write'), {
                    mayRead: true,
                    mayWrite: true,
                    mayManage: false,
                    userId: userB.identity,
                    realmUrl: "/~/products",
                });
                yield assertUserPermissions(userA, {
                    userId: userA.identity,
                    mayRead: true,
                    mayWrite: true,
                    mayManage: true,
                    path: `/${userA.identity}/products`
                });
                yield assertUserPermissions(userB, {
                    userId: userB.identity,
                    mayRead: true,
                    mayWrite: true,
                    mayManage: false,
                    path: `/${userA.identity}/products`
                });
            }));
        });
        describe("userA granting admin to userB", () => {
            it("should be successful", () => __awaiter(this, void 0, void 0, function* () {
                assertAppliedPermissions(yield userA.applyPermissions({ userId: userB.identity }, "/~/products", 'admin'), {
                    mayRead: true,
                    mayWrite: true,
                    mayManage: true,
                    userId: userB.identity,
                    realmUrl: "/~/products",
                });
                yield assertUserPermissions(userA, {
                    userId: userA.identity,
                    mayRead: true,
                    mayWrite: true,
                    mayManage: true,
                    path: `/${userA.identity}/products`
                });
                yield assertUserPermissions(userB, {
                    userId: userB.identity,
                    mayRead: true,
                    mayWrite: true,
                    mayManage: true,
                    path: `/${userA.identity}/products`
                });
            }));
        });
        describe("userA granting write to userB with userC's Realm", () => {
            it("should fail", () => __awaiter(this, void 0, void 0, function* () {
                const userC = yield Realm.Sync.User.register(server.url, faker.internet.userName(), faker.internet.password());
                const userCRealm = yield Realm.open({
                    sync: {
                        url: `realm://${server.address}/~/products`,
                        user: userC,
                    },
                    schema: [ProductSchema]
                });
                const err = yield chai_1.assert.isRejected(userA.applyPermissions({ userId: userB.identity }, `/${userC.identity}/products`, 'write'), "The path is invalid or current user has no access.");
                chai_1.assert.equal(err.statusCode, 614);
            }));
        });
        describe("userA granting admin to userB who opens a partial sync of it", () => {
            it("should succeed", () => __awaiter(this, void 0, void 0, function* () {
                assertAppliedPermissions(yield userA.applyPermissions({ userId: userB.identity }, "/~/products", 'admin'), {
                    mayRead: true,
                    mayWrite: true,
                    mayManage: true,
                    userId: userB.identity,
                    realmUrl: "/~/products",
                });
                yield chai_1.assert.isFulfilled(Realm.open({
                    sync: {
                        url: `realm://${server.address}/${userA.identity}/products/__partial/1`,
                        user: userB,
                    },
                    schema: [ProductSchema]
                }));
            }));
        });
        describe("userB opens userA's Realm partially without permission", () => {
            it("should fail", () => __awaiter(this, void 0, void 0, function* () {
                const err = yield chai_1.assert.isRejected(new Promise((resolve, reject) => {
                    Realm.open({
                        sync: {
                            url: `realm://${server.address}/${userA.identity}/products/__partial/1`,
                            user: userB,
                            error: (session, err) => reject(err)
                        },
                        schema: [ProductSchema]
                    }).then(() => resolve()),
                        "The path is invalid or current user has no access.";
                }));
                chai_1.assert.equal(err.status, 403);
            }));
        });
        describe("userA granting read to all", () => {
            it("should be successful for userB to open", () => __awaiter(this, void 0, void 0, function* () {
                const response = yield userA.applyPermissions({ userId: "*" }, "/~/products", 'read');
                yield chai_1.assert.isFulfilled(Realm.open({
                    sync: {
                        url: `realm://${server.address}/${userA.identity}/products`,
                        user: userB,
                    },
                    schema: [ProductSchema]
                }));
            }));
        });
        describe("userA granting admin to all", () => {
            it("should allow userB to change permission to write", () => __awaiter(this, void 0, void 0, function* () {
                const response = yield userA.applyPermissions({ userId: "*" }, "/~/products", 'admin');
                assertAppliedPermissions(yield userB.applyPermissions({ userId: "*" }, `/${userA.identity}/products`, 'write'), {
                    mayRead: true,
                    mayWrite: true,
                    mayManage: false,
                    userId: "*",
                    realmUrl: `/${userA.identity}/products`,
                });
            }));
        });
    });
});
//# sourceMappingURL=realmjs-integration-tests.spec.js.map